@@ -32,17 +32,20 @@ new core of the Bombardier server."""
 __author__ =  'Peter Banka'
 __version__ = '1.0'
 
+import BreakHandler
 import StringIO
 import urllib2, urlparse, os, time
 import yaml
 import pycurl
+import exceptions
+import re
 import urllib
 from bombardier_core.static_data import OK, FAIL
 from Exceptions import UnexpectedDataException, ServerException
 from Exceptions import MachineTraceback, ServerTracebackException
 from Exceptions import MachineUnavailableException
 import libUi
-from SystemStateSingleton import SystemState, ENABLE
+from SystemStateSingleton import SystemState
 system_state = SystemState()
 
 LOGIN_PATH = "/accounts/login/"
@@ -118,7 +121,7 @@ class Response:
             except yaml.parser.ParserError:
                 # New Django JSON is not compatible with yaml. Massage it.
                 try:
-                    new_output = ''.join(str(self.output).split())
+                    new_output = ''.join(str(self.output).split('\n'))
                     output = yaml.load(new_output)
                 except yaml.parser.ParserError:
                     msg = "can't convert to yaml (%s)" % self.output
@@ -143,6 +146,7 @@ class CnmConnector:
         '''
         address -- URL of the Bombardier web server
         username -- username for connecting to the server'''
+        self.break_handler = BreakHandler.BreakHandler()
         self.address = address
         self.username = username
         self.password = None
@@ -150,7 +154,8 @@ class CnmConnector:
         self.proxy_port = None
         self.debug = False
         self.session_id = None
-        self.cookie_file = os.path.join(os.environ.get("HOME"), '.bom_cookie.txt')
+        self.cookie_file = os.path.join(os.environ.get("HOME"),
+                                        '.bom_cookie.txt')
 
     def prepare_curl_object(self, url):
         '''prepares pycurl object that can be used to communicate to the
@@ -165,7 +170,7 @@ class CnmConnector:
         curl_obj.setopt(pycurl.COOKIEFILE, self.cookie_file)
         curl_obj.setopt(pycurl.COOKIEJAR, self.cookie_file)
         if self.debug:
-            print "CONNECTING TO: ",url
+            print "CONNECTING TO: ", url
         if self.username and self.password:
             auth_string = "%s:%s" % (self.username, self.password)
             curl_obj.setopt(pycurl.USERPWD, auth_string)
@@ -191,9 +196,11 @@ class CnmConnector:
         if self.debug:
             print "PERFORMING:", curl_obj
         try:
+            self.break_handler.enable()
             curl_obj.perform()
             http_code = curl_obj.getinfo(pycurl.HTTP_CODE)
             response.set_http_code(http_code)
+            self.break_handler.disable()
             return response
         except pycurl.error, curl_err:
             http_code = curl_obj.getinfo(pycurl.HTTP_CODE)
@@ -226,7 +233,8 @@ class CnmConnector:
         return return_data.get("super_user")
 
     def service_request(self, path, args=None,
-                        put_data=None, post_data=None, timeout=None):
+                        put_data=None, post_data=None,
+                        delete=False, timeout=None):
         '''Performs a GET or a POST or a PUT, depending on data provided
         and returns text data based on output
         path -- relative path
@@ -253,6 +261,8 @@ class CnmConnector:
             curl_obj.setopt(pycurl.READFUNCTION, out_data.read)
             if self.debug:
                 print "Setting PUT data"
+        if delete:
+            curl_obj.setopt(pycurl.CUSTOMREQUEST, "DELETE")
         if post_data != None:
             post_list = []
             for key in post_data:
@@ -265,64 +275,113 @@ class CnmConnector:
         return self.perform_request(curl_obj, full_path)
 
     def sync_server_home(self, server_home_path):
+        "For setting the base location of all data on the CNM"
         url = '/json/server/config'
-        post_data={"server_home": server_home_path}
+        post_data = {"server_home": server_home_path}
         output = self.service_yaml_request(url, post_data=post_data)
         new_path = output["server_home"]
         if new_path != server_home_path:
             raise UnexpectedDataException("server_home is set to %s" % new_path)
 
     def service_yaml_request(self, path, args=None,
-                             put_data=None, post_data=None, timeout=None):
+                             put_data=None, post_data=None,
+                             delete=False, timeout=None):
         '''same as service_request, but assumes that return data from the server
-        is YAML(JSON) and converts it to return a python dictionary instead of text.'''
+        is YAML(JSON) and converts it to return a python dictionary instead of
+        text.'''
         try:
             if put_data != None:
                 if type(put_data) == type(["list"]) or \
                    type(put_data) == type({}):
                     put_data = yaml.dump(put_data)
-            response = self.service_request(path, args, put_data, post_data, timeout)
+            response = self.service_request(path, args, put_data, post_data,
+                                            delete, timeout)
             return response.convert_from_yaml()
         except urllib2.HTTPError:
             print "Unable to connect to the service %s" % path
             return {}
 
     def cleanup_connections(self):
+        "Call the ReST interface for clearing connections no a machine"
         url = "json/machine/cleanup_connections"
         self.service_yaml_request(url, post_data={})
 
     def _is_any_alive(self, poll_output):
+        "Determine if anu jobs are alive from what the CNM told us"
         for job_name in poll_output:
             if poll_output[job_name].get("alive", False):
                 return True
         return False
 
     def watch_jobs(self, job_names):
-        libUi.info("Job names: %s" % job_names)
-        summary_output = {}
-        while job_names:
-            time.sleep(0.25)
-            url = "json/job/poll/"
-            post_data = {"yaml": yaml.dump({"job_names": job_names})}
-            output = system_state.cnm_connector.service_yaml_request(url, post_data=post_data)
-            for job_name in output:
-                job_output = output[job_name]
-                if "traceback" in job_output:
-                    print "TRACEBACK"
-                    raise MachineTraceback(url, job_output["traceback"]) #FIXME what happens here?
-                new_output = job_output.get("new_output", '')
-                if new_output:
-                    libUi.process_cnm(new_output)
-                if not job_output.get("alive", False):
-                    libUi.info("Joining %s..." % job_name)
-                    out = system_state.cnm_connector.join_job(job_name)
-                    summary_output[job_name] = out
-                    job_names +=  out.get("children", [])
-                    job_names.remove(job_name)
-                 
-        return summary_output
+        "Given a list of jobs, watch their progress."
+        summary_output = []
+        summary_status = OK
+        jobs = set(job_names)
+        try:
+            while job_names:
+                jobs = jobs.union(job_names)
+                time.sleep(0.25)
+                url = "json/job/poll/"
+                post_data = {"yaml": yaml.dump({"job_names": job_names})}
+                output = self.service_yaml_request(url, post_data=post_data)
+                if type(output) != type({}):
+                    msg = "Request to %s returned and invalid type (%s)"
+                    libUi.error(msg % (url, output))
+                    return FAIL, output
+                for job_name in output:
+                    job_output = output[job_name]
+                    if "traceback" in job_output:
+                        raise MachineTraceback(url, job_output["traceback"])
+                    new_output = job_output.get("new_output", '')
+                    if new_output:
+                        libUi.process_cnm(new_output)
+                    alive = job_output.get("alive")
+                    if alive == None:
+                        if job_output.get("command_status") == "QUEUED":
+                            pending_job = job_output.get("pending_job")
+                            msg = "%s is pending %s." % (job_name, pending_job)
+                            libUi.warning(msg)
+                            libUi.info("Watching %s..." % pending_job)
+                            job_names = [pending_job]
+                            break
+                    if alive == False:
+                        out = system_state.cnm_connector.join_job(job_name)
+                        summary_output.append(out.get("command_output"))
+                        status = out.get("command_status", OK)
+                        if status != OK:
+                            summary_status = FAIL
+                        job_names +=  out.get("children", [])
+                        job_names.remove(job_name)
+                        comments = out.get("jobs_requiring_comment", 0)
+                        if comments:
+                            system_state.comment_counter = len(comments)
+        except exceptions.KeyboardInterrupt:
+            term = libUi.ask_yes_no("Terminate job", libUi.YES)
+            if term == libUi.NO:
+                msg = ["Disconnected from %s" % job_names,
+                       "To re-connect, use 'job view'"]
+                return OK, msg
+            else:
+                matcher = re.compile("(\S+)@(\S+)\-(\d+)")
+                machine_names = set()
+                for job_name in job_names:
+                    _user, machine_name, _counter = matcher.findall(job_name)[0]
+                    machine_names.update([machine_name])
+                output = {}
+                for machine_name in machine_names:
+                    url = "/json/machine/stop-jobs/"
+                    post_data = {"machine": machine_name}
+                    print ">>> Halting all jobs on %s..." % machine_name
+                    output = self.service_yaml_request(url, post_data=post_data)
+                    libUi.user_output(output["command_output"], FAIL)
+                raise exceptions.KeyboardInterrupt
+
+        libUi.info("Finishing processing of %d jobs..." % len(jobs))
+        return summary_status, summary_output
 
     def get_job(self, url, post_data):
+        "Given an action dictated by URL, get a job_name that we can track"
         out = self.service_yaml_request(url, post_data=post_data)
         if "traceback" in out:
             raise MachineTraceback(url, out["traceback"])
@@ -332,19 +391,22 @@ class CnmConnector:
         return job_name
 
     def join_job(self, job_name):
+        "We think a job's finished. Let's check in with the server"
         url = "json/job/join/%s" % job_name
         output = self.service_yaml_request(url)
         if "traceback" in output:
-            raise MachineTraceback(url, out["traceback"])
+            raise MachineTraceback(url, output["traceback"])
         return output
 
     def set_password(self, password):
+        "Set the configuration-key"
         url = "/json/dispatcher/set-password"
         post_data = {"password": password}
         output = self.service_yaml_request(url, post_data=post_data)
         return output["command_status"], output["command_output"]
 
     def dispatcher_control(self, action, post_data = {}):
+        "Performing actions on the dispatcher"
         dispatcher_url = "/json/dispatcher/%s" % action
         if action == "status":
             output = self.service_yaml_request(dispatcher_url)
@@ -366,3 +428,21 @@ class CnmConnector:
                 else:
                     raise
         return output["command_status"], output["command_output"]
+
+    def get_uncommented_jobs(self):
+        "get a list of tuples of jobs that need commenting"
+        url = '/json/job/comment/pending/'
+        content_dict = self.service_yaml_request(url)
+        return content_dict["job_info"]
+
+    def post_comments(self, publish, job_names, comments):
+        "Attach comments to a list of job names"
+        data = {"job_names": job_names,
+                "comment": comments,
+                "publish": publish,
+               }
+        post_data = {"yaml": yaml.dump(data)}
+        url = '/json/job/comment/pending/'
+        content_dict = self.service_yaml_request(url, post_data=post_data)
+        return content_dict
+
