@@ -31,6 +31,8 @@
 Supports access to 'configurable items' on the bombardier web server'''
 
 import PinshCmd
+import random
+import yaml
 from bombardier_core.static_data import OK, FAIL, PARTIAL, COMPLETE, NO_MATCH
 from Exceptions import UnexpectedDataException
 from SystemStateSingleton import SystemState
@@ -63,10 +65,19 @@ DEFAULT_BOM = """
 -
 """
 
+DEFAULT_USER = """email: user@example.com
+first_name: ''
+is_active: true
+is_superuser: false
+last_name: ''
+password: '%s'
+"""
+
 class ConfigField(PinshCmd.PinshCmd):
     '''The server keeps track of several types of configuration data.
     This class can provide command-line completion for those data objects'''
-    def __init__(self, name = "configField", data_type=MERGED, strict=True, new=False):
+    def __init__(self, name = "configField", data_type=MERGED, strict=True,
+                 new=False, machine_field=None):
         '''
         name -- Not used except in debugging
         data_type -- which type of configuration data is this object supposed
@@ -82,6 +93,7 @@ class ConfigField(PinshCmd.PinshCmd):
         self.data_type = data_type
         self.cmd_owner = 0
         self.strict = strict # take only exact matches
+        self.machine_field = machine_field
         if data_type == MACHINE:
             self.directory = "machine"
         elif data_type == MERGED:
@@ -113,6 +125,13 @@ class ConfigField(PinshCmd.PinshCmd):
             object_list = [ x.get("fields").get("name") for x in data ]
         return object_list
 
+    def post_data(self, first_token_name, data):
+        'posts data to modify the object'
+        post_data = {"yaml": yaml.dump(data)}
+        url = "json/%s/name/%s" % (self.directory, first_token_name)
+        output = system_state.cnm_connector.service_yaml_request(url, post_data=post_data)
+        return output
+
     def get_data(self, first_token_name):
         'returns a list of all configuration objects of this type'
         url = "json/%s/name/%s" % (self.directory, first_token_name)
@@ -155,6 +174,7 @@ class ConfigField(PinshCmd.PinshCmd):
             return '' # EXPERIMENTAL CHANGE from []
         return data
 
+
     def post_specific_data(self, tokens, index, new_data):
         '''used with the edit command to upload data to server'''
         tokens[index] = tokens[index].replace('"', '')
@@ -180,7 +200,18 @@ class ConfigField(PinshCmd.PinshCmd):
         will return 'localhost' for the machine name if strict is off,
         otherwise, it will return 'localh'.
         '''
-        #print "PN: begin self.string = ", self.strict
+        if self.machine_field:
+            machine_name = tokens[self.machine_field]
+            url = "json/machine/name/%s" % machine_name
+            machine_config = system_state.cnm_connector.service_yaml_request(url)
+            possible_names = machine_config.get(self.directory, [])
+            output_names = []
+            for name in possible_names:
+                if name.lower().startswith(tokens[-1].lower()):
+                    output_names.append(name)
+            return output_names
+            
+        #print "PN: begin tokens, index = ", tokens, index
         tokens[index] = tokens[index].replace('"', '')
         if not self.strict:
             return tokens[index:]
@@ -217,6 +248,11 @@ class ConfigField(PinshCmd.PinshCmd):
     def get_default_data(self):
         if self.data_type == MACHINE:
             return DEFAULT_MACHINE
+        elif self.data_type == USER:
+            letters = [ chr( x ) for x in range(65, 91) ]
+            random.shuffle(letters)
+            default_user = DEFAULT_USER % ''.join(letters)
+            return default_user
         elif self.data_type == BOM:
             return DEFAULT_BOM
         elif self.data_type == PACKAGE:
